@@ -74,13 +74,13 @@ def exercise(file_name, out = sys.stdout):
   # Make sure this model has 126 sites (42 sites times 3-fold ncs)
   assert ncs_mam.model().get_sites_cart().size() == 126
   assert approx_equal (ncs_mam.model().get_sites_cart()[0],
-    (23.560999999999996, 8.159, 10.660000000000002))
+    (23.560999999999996, 8.159, 10.660000000000002), 1e-4)
 
   # Get just unique part (42 sites)
   unique_mam=ncs_mam.extract_all_maps_around_model(select_unique_by_ncs=True)
   assert unique_mam.model().get_sites_cart().size() == 42
   assert approx_equal (unique_mam.model().get_sites_cart()[0],
-    (18.740916666666664, 13.1794, 16.10544))
+    (18.740916666666664, 13.1794, 16.10544), 1e-3)
 
   # Make sure that the extraction did not change the original but does change
   #   the extracted part
@@ -90,7 +90,7 @@ def exercise(file_name, out = sys.stdout):
   # Shift back the extracted part and make sure it matches the original now
   shifted_back_unique_model=mmm.get_model_from_other(unique_mam.deep_copy())
   assert approx_equal (shifted_back_unique_model.get_sites_cart()[0],
-    (23.560999999999996, 8.158999999999997, 10.66))
+    (23.560999999999996, 8.158999999999997, 10.66), 1e-4)
 
   # Change the extracted model
   sites_cart=unique_mam.model().get_sites_cart()
@@ -109,9 +109,9 @@ def exercise(file_name, out = sys.stdout):
     other_model_id = 'model')
   # ...and check that copy 1 and copy 2 both change
   assert approx_equal (ncs_mam.model().get_sites_cart()[0],
-     (5.820083333333333, -4.020400000000001, -4.445440000000001))
+     (5.820083333333333, -4.020400000000001, -4.445440000000001), 1e-3)
   assert approx_equal (ncs_mam.model().get_sites_cart()[42],
-     (38.41904613024224, 17.233251085893276, 2.5547442135142524))
+     (38.41904613024224, 17.233251085893276, 2.5547442135142524), 1e-3)
 
   # Find ncs from map or model
   nn=ncs_mam_copy
