@@ -4,6 +4,7 @@ import sys
 import libtbx.load_env
 
 tst_list_base = [
+  "$D/regression/tst_mrc_io.py",
   "$D/regression/tst_wildcard.py",
   "$D/gui_tools/tst.py",
   "$D/regression/tst_simple_parser.py",
@@ -17,10 +18,15 @@ tst_list_base = [
   "$D/regression/tst_map_model_manager_3.py",
   "$D/regression/tst_map_model_manager_4.py",
   "$D/regression/tst_map_model_manager_model_sharpening_5.py",
+  "$D/regression/tst_map_model_manager_model_sharpening_5_cif.py",
   "$D/regression/tst_map_model_manager_call_consistency.py",
   "$D/regression/tst_map_model_manager_external_sharpening_7.py",
   "$D/regression/tst_map_model_manager_half_map_sharpening_6.py",
   "$D/regression/tst_map_model_manager_tls_from_map_8.py",
+  "$D/regression/tst_map_model_manager_cif.py",
+  "$D/regression/tst_map_model_manager_9_remove_origin_shift_and_unit_cell_crystal_symmetry.py",
+  "$D/regression/tst_map_model_manager_9_remove_origin_shift_and_unit_cell_crystal_symmetry_cif.py",
+  "$D/regression/tst_map_model_manager_local_resolution_10.py",
   "$D/regression/tst_map_tools.py",
   "$D/regression/tst_patterson.py",
   "$D/regression/tst_restraints_merge.py",
@@ -62,6 +68,7 @@ tst_list_base = [
   "$D/pdb/modified_rna_dna_names.py",
   "$D/regression/secondary_structure/tst_sheet.py",
   "$D/regression/secondary_structure/tst_annotation.py",
+  "$D/regression/secondary_structure/tst_annotation_long.py",
   "$D/pdb/secondary_structure.py",
   "$D/pdb/tst_atom_selection_string.py",
   "$D/pdb/tst_secondary_structure.py",
@@ -98,7 +105,8 @@ tst_list_base = [
   "$D/regression/tst_bioinformatics.py",
   "$D/regression/tst_box_around_molecule.py",
   "$D/regression/tst_mmcif_segids.py",
-  "$D/regression/tst_hierarchy_pdb_v3.py",
+  "$D/regression/tst_mmcif_input.py",
+  "$D/regression/tst_hierarchy_forward_compatible_pdb.py",
   "$D/regression/tst_mmcif_multimodel.py",
   "$D/regression/tst_add_conformations.py",
   "$D/regression/tst_symmetry.py",
@@ -122,6 +130,7 @@ tst_list_base = [
   "$D/regression/ncs/tst_mmcif_biomt_reduction_output.py",
   "$D/regression/ncs/tst_ncs_search_ligs.py",
   "$D/regression/ncs/tst_ncs_search_broken_chain.py",
+  "$D/regression/ncs/tst_ncs_search_shortcut_1.py",
   "$D/regression/ncs/tst_ncs_groups_preprocessing.py",
   "$D/regression/ncs/tst_ncs_input.py",
   "$D/regression/ncs/tst_ncs_user_selections.py",
@@ -145,6 +154,7 @@ tst_list_base = [
   "$D/regression/tst_mtz_as_cif.py",
   "$D/regression/tst_group_rounding.py",
   "$D/regression/tst_hierarchy_occupancies_rounding.py",
+  "$D/regression/tst_hierarchy_merge_atoms_at_end_to_residues.py",
   "$D/regression/tst_hierarchy_long_chain_ids_1.py",
   "$D/regression/tst_hierarchy_long_resname_1.py",
   "$D/regression/tst_hierarchy_long_resname_2.py",
@@ -152,6 +162,7 @@ tst_list_base = [
   "$D/regression/tst_hierarchy_long_resname_4.py",
   "$D/regression/tst_hierarchy_copy_select.py",
   "$D/regression/tst_hierarchy_id_str.py",
+  "$D/regression/tst_hierarchy_altlocs.py",
   "$D/regression/tst_fetch.py",
   ]
 
