@@ -276,13 +276,9 @@ def mahalanobis_using_sklearn(x=None, data=None, cov=None, verbose=False):
   if verbose: print(mahal_cov)
   return mahal_cov
 
-def covariance_using_sklearn(data=None, values=None, verbose=False):
+def covariance_using_sklearn(data=None, choice='Empirical', return_numpy=True, verbose=False):
   from sklearn.covariance import EmpiricalCovariance, MinCovDet
-  if values:
-    assert 0
-    cov=EmpiricalCovariance()
-    cov.covariance_=values
-    return cov
+  import numpy as np
   # fit a MCD robust estimator to data
   robust_cov = MinCovDet().fit(data)
   # fit a MLE estimator to data
@@ -294,22 +290,92 @@ def covariance_using_sklearn(data=None, values=None, verbose=False):
         )
     )
   # choose one
-  cov = emp_cov
+  if choice=='Empirical':
+    cov = emp_cov
+  elif choice=='Robust':
+    cov = robust_cov
+  if return_numpy:
+    return np.asarray(cov.covariance_)
   return cov
 
-def mahalanobis_p_values_outlier_indices(x=None, data=None, cov=None):
-  rc = mahalanobis_p_values(x=x, data=data, cov=cov)
+def covariance_using_numpy(data):
+  import numpy as np
+  return np.cov(np.transpose(data))
+
+def numpy_mean(data, index=0):
+  import numpy as np
+  return np.mean(data, index)
+
+def mahalanobis_using_numpy(x, data=None, mu=None, cov=None, inv_cov=None, verbose=False):
+  """
+  Compute the Mahalanobis Distance between each row of x and the data
+    x    : vector or matrix of data with, say, p columns.
+    data : ndarray of the distribution from which Mahalanobis distance of each
+           observation of x is to be computed.
+    mu   : mean of data. If None, computer from data.
+    cov  : covariance matrix (p x p) of the distribution. If None, computed from data.
+    inv_cov : inverse of covariance matrix. If None, computed from cov
+  """
+  import numpy as np
+  def _mahalanobis(x, data=None, mu=None, cov=None, inv_cov=None):
+    if verbose:
+      print('x',x)
+      print('data',data)
+    if mu is None:
+      mu = np.mean(data, 0)
+    else:
+      mu = np.asarray(mu)
+    if verbose: print('mu',mu)
+    if inv_cov is None:
+      if cov is None:
+        cov = covariance_using_numpy(data)
+      else:
+        cov=np.asarray(cov)
+      inv_cov = np.linalg.inv(cov)
+    if verbose:
+      print('Covariance')
+      print(cov)
+    if verbose:
+      print('Inverse Covariance')
+      print(inv_cov)
+    x_minus_mn = x - mu
+    if verbose:
+      print('x_minus_mn')
+      print(x_minus_mn)
+    D_square = np.dot(np.dot(x_minus_mn, inv_cov), np.transpose(x_minus_mn))
+    if verbose:
+      print('D_square')
+      print(D_square)
+      print(D_square.diagonal())
+    return D_square.diagonal()
+
+  if verbose:
+    print(x)
+    print(data)
+    print(mu)
+    print(cov)
+    print(inv_cov)
+  if data is None:
+    assert mu and (cov is not None or inv_cov is not None)
+  rc = _mahalanobis(x, data=data, mu=mu, cov=cov, inv_cov=inv_cov)
+  return rc
+
+def mahalanobis_p_values_outlier_indices(x=None, data=None, mu=None, cov=None, inv_cov=None, verbose=False):
+  rc = mahalanobis_p_values(x=x, data=data, mu=mu, cov=cov, inv_cov=inv_cov, verbose=verbose)
   return list(filter(lambda x: rc[x] <0.01, range(len(rc))))
 
-def mahalanobis_p_values(x=None, data=None, cov=None, verbose=False):
+def mahalanobis_p_values(x=None, data=None, mu=None, cov=None, inv_cov=None, verbose=False):
   from scipy.stats import chi2
   df=len(x[0])-1
   if verbose:
     print(chi2.ppf((1-0.01), df=df))
-    #> 9.21
-  mahal_cov = mahalanobis_using_sklearn(x=x, data=data, cov=cov)
+    #> 9.21 for df=2
+  mahal_cov = mahalanobis(x=x, data=data, mu=mu, cov=cov, inv_cov=inv_cov, verbose=verbose)
   if verbose: print(mahal_cov)
   return(1 - chi2.cdf(mahal_cov, df))
 
-def mahalanobis(x=None, data=None, mu=None, cov=None):
-  return mahalanobis_using_sklearn(x=x, data=data, cov=cov)
+def mahalanobis(x=None, data=None, mu=None, cov=None, inv_cov=None, verbose=False):
+  if 1:
+    return mahalanobis_using_numpy(x=x, data=data, mu=mu, cov=cov, inv_cov=inv_cov, verbose=verbose)
+  else:
+    return mahalanobis_using_sklearn(x=x, data=data, cov=cov, verbose=verbose)
