@@ -1,6 +1,7 @@
 from __future__ import absolute_import, division, print_function
 import six
-import os, sys, time
+import sys, time
+from libtbx.utils import Sorry
 import mmtbx.model
 import iotbx.pdb
 import boost_adaptbx.boost.python as bp
@@ -20,15 +21,9 @@ get_class = iotbx.pdb.common_residue_names_get_class
 def get_h_restraints(resname, strict=True):
   from mmtbx.monomer_library import cif_types
   from mmtbx.chemical_components import get_cif_dictionary
-  from mmtbx.chemical_components import get_cif_filename
-  from mmtbx.ligands.rdkit_utils import read_chemical_component_filename
-  filename = get_cif_filename(resname)
-  if not os.path.exists(filename): return None
-  try:
-    molecule = read_chemical_component_filename(filename)
-  except Exception as e:
-    print(e)
-    return None
+  from mmtbx.ligands.rdkit_utils import get_molecule_from_resname
+  molecule = get_molecule_from_resname(resname)
+  if molecule is None: return None
   cc_cif = get_cif_dictionary(resname)
   cc = cc_cif['_chem_comp'][0]
   hs = []
@@ -38,9 +33,9 @@ def get_h_restraints(resname, strict=True):
     three_letter_code=cc.three_letter_code,
     name=cc.name,
     group=cc.type,
-    # number_atoms_all=cc.number_atoms_all,
-    # number_atoms_nh=cc.number_atoms_nh,
-    desc_level="")
+    number_atoms_all=0, #cc.number_atoms_all,
+    number_atoms_nh=0, #cc.number_atoms_nh,
+    desc_level=".")
   comp_comp_id = cif_types.comp_comp_id(source_info=None, chem_comp=chem_comp)
   lookup = {}
   for i, a in enumerate(cc_cif.get('_chem_comp_atom',[])):
@@ -62,6 +57,8 @@ def get_h_restraints(resname, strict=True):
     if strict:
       if (b.atom_id_1 not in hs and
           b.atom_id_2 not in hs): continue
+    if ( b.atom_id_1 not in lookup or
+         b.atom_id_2 not in lookup): continue
     atom_idx1=lookup[b.atom_id_1]
     atom_idx2=lookup[b.atom_id_2]
     bl = rdMolTransforms.GetBondLength(conf, atom_idx1, atom_idx2)
@@ -74,9 +71,14 @@ def get_h_restraints(resname, strict=True):
 
   from mmtbx.ligands.rdkit_utils import enumerate_angles
   for angle in enumerate_angles(molecule):
-    if angle[0] in hsi or angle[2] in hsi:
+    if strict:
+      if angle[0] in hsi or angle[2] in hsi:
+        av = rdMolTransforms.GetAngleDeg(conf, angle[0], angle[1], angle[2])
+      else: continue
+    else:
       av = rdMolTransforms.GetAngleDeg(conf, angle[0], angle[1], angle[2])
-    else: continue
+    if ( angle[0] not in lookup or
+         angle[2] not in lookup): continue
     comp_comp_id.angle_list.append(cif_types.chem_comp_angle(
       atom_id_1=lookup[angle[0]],
       atom_id_2=lookup[angle[1]],
@@ -86,9 +88,14 @@ def get_h_restraints(resname, strict=True):
 
   from mmtbx.ligands.rdkit_utils import enumerate_torsions
   for i, angle in enumerate(enumerate_torsions(molecule)):
-    if angle[0] in hsi or angle[3] in hsi:
+    if strict:
+      if angle[0] in hsi or angle[3] in hsi:
+        av = rdMolTransforms.GetDihedralDeg(conf, angle[0], angle[1], angle[2], angle[3])
+      else: continue
+    else:
       av = rdMolTransforms.GetDihedralDeg(conf, angle[0], angle[1], angle[2], angle[3])
-    else: continue
+    if ( angle[0] not in lookup or
+         angle[3] not in lookup): continue
     comp_comp_id.tor_list.append(cif_types.chem_comp_tor(
       id='Var_%03d' % i,
       atom_id_1=lookup[angle[0]],
@@ -112,10 +119,26 @@ def mon_lib_query(residue, mon_lib_srv, construct_h_restraints=True):
   md, ani = mon_lib_srv.get_comp_comp_id_and_atom_name_interpretation(
     residue_name=residue.resname,
     atom_names=residue.atoms().extract_name())
+  cif_object=None
+  # if md is None:
+  #   md, ani = mon_lib_srv.get_comp_comp_id_and_atom_name_interpretation(
+  #     residue_name='%s_EL' % residue.resname,
+  #     atom_names=residue.atoms().extract_name(),
+  #     ad_hoc_single_atom_residues=True)
   if md is None:
-    md = get_h_restraints(residue.resname)
+    md = get_h_restraints(residue.resname, strict=False)
     # md.show()
-  return md
+    if md is None:
+      raise Sorry('Entity "%s" not found in CCD (or GeoStd). Please supply restraints.' % residue.resname)
+    from six.moves import cStringIO as StringIO
+    input_string='data_comp_list\n'
+    input_string+=str(md.chem_comp.as_cif_loop())
+    f=StringIO()
+    md.show(f=f)
+    input_string += '\ndata_comp_%s\n' % residue.resname
+    input_string += '\n%s' % f.getvalue()
+    cif_object = iotbx.cif.reader(input_string=input_string).model()
+  return md, cif_object
 
 # ==============================================================================
 
@@ -196,7 +219,8 @@ class place_hydrogens():
       self.time_remove_isolated    = None
       self.time_riding_manager     = None
       self.time_remove_H_nopara    = None
-      self.time_reset_idealize     = None
+      self.time_reset              = None
+      self.time_idealize           = None
       self.time_remove_H_on_links  = None
 
 # ------------------------------------------------------------------------------
@@ -271,7 +295,8 @@ class place_hydrogens():
       crystal_symmetry  = self.model.crystal_symmetry(),
       restraint_objects = ro,
       log               = null_out())
-    self.model.process(pdb_interpretation_params=p, make_restraints=True)
+    self.model.process(pdb_interpretation_params=p,
+                       make_restraints=True)
     #self.model.idealize_h_minimization()
     #STOP()
     self.time_make_grm = round(time.time()-t0, 2)
@@ -336,8 +361,10 @@ class place_hydrogens():
     t0 = time.time()
     self.model.reset_adp_for_hydrogens(scale = self.adp_scale)
     self.model.reset_occupancy_for_hydrogens_simple()
+    self.time_reset = round(time.time()-t0, 2)
+    t0 = time.time()
     self.model.idealize_h_riding()
-    self.time_reset_idealize = round(time.time()-t0, 2)
+    self.time_idealize = round(time.time()-t0, 2)
 
     # Remove H atoms that are involved in links (bonds, metal coordination, etc)
     # --------------------------------------------------------------------------
@@ -419,14 +446,36 @@ class place_hydrogens():
             if(get_class(name=ag.resname) == "common_water"): continue
             actual = [a.name.strip().upper() for a in ag.atoms()]
             #
-            mlq = mon_lib_query(residue=ag, mon_lib_srv=mon_lib_srv)
+            mlq, cif_object = mon_lib_query(residue=ag, mon_lib_srv=mon_lib_srv)
             if mlq is None:
               self.no_H_placed_mlq.append(ag.resname)
               continue
 
+            if cif_object:
+              ro = self.model.get_restraint_objects()
+              if ro is None: ro=[]
+              ro.append(('auto_%s' % ag.resname, cif_object))
+              self.model.set_restraint_objects(ro)
+
             expected_h = []
             #expected_ha = []
             atom_dict = mlq.atom_dict()
+            def _remove_atoms(atom_dict, names):
+              remove=[]
+              for k,v in atom_dict.items():
+                if k in names:
+                  remove.append(k)
+              if remove:
+                for r in remove:
+                  del atom_dict[r]
+              return atom_dict
+            #
+            # don't add polymer H atoms. Terminal H atoms added elsewhere
+            #
+            if mlq.test_for_peptide(atom_dict):
+              atom_dict = _remove_atoms(atom_dict, ['H2', 'HXT'])
+            elif mlq.test_for_rna_dna(atom_dict):
+              atom_dict = _remove_atoms(atom_dict, ["HO3'", 'HO3*'])
             for k, v in six.iteritems(atom_dict):
               if(v.type_symbol=="H"):
                 expected_h.append(k)
@@ -658,7 +707,8 @@ The following H atoms were not placed because they could not be parameterized
       time_remove_isolated    = self.time_remove_isolated,
       time_riding_manager     = self.time_riding_manager,
       time_remove_H_nopara    = self.time_remove_H_nopara,
-      time_reset_idealize     = self.time_reset_idealize,
+      time_reset              = self.time_reset,
+      time_idealize           = self.time_idealize,
       time_remove_H_on_links  = self.time_remove_H_on_links)
 
 # ------------------------------------------------------------------------------
@@ -667,13 +717,14 @@ The following H atoms were not placed because they could not be parameterized
     print('Detailed timings:')
     print("Rebox model:", self.time_rebox_model)
     print('Remove element X:', self.time_remove_element_X)
-    print("Add_missing_H_atoms_at_bogus_position:", self.time_add_missing_H)
-    print('Add N-terminal propeller', self.time_terminal_propeller)
+    print("Add missing H at bogus position:", self.time_add_missing_H)
+    print('Add N-terminal propeller:', self.time_terminal_propeller)
     print("Get new model obj and grm:", self.time_make_grm )
     print("Remove isolated H:", self.time_remove_isolated)
     print("Setup Riding manager:", self.time_riding_manager)
     print("Remove H that were not parameterized:", self.time_remove_H_nopara)
-    print("Reset adp, occ; idealize H positions:", self.time_reset_idealize)
+    print("Reset adp, occ:", self.time_reset)
+    print("idealize H positions:", self.time_idealize)
     print("Remove H on links:", self.time_remove_H_on_links)
     print()
 
