@@ -290,20 +290,18 @@ class compute_fo_minus_fo_map(object):
       if (self.silent) : peak_search_log = null_out()
       fmodel = self.fmodel
       peaks = find_peaks.manager(
-        fmodel=fmodel,
-        map_type=None,
-        map_coeffs=self.map_coeff,
-        map_cutoff=self.map_cutoff,
-        params=self.peak_search_params,
-        log=peak_search_log).peaks_mapped()
+        map_cutoff     = self.map_cutoff,
+        xray_structure = fmodel.xray_structure,
+        params         = self.peak_search_params,
+        log            = peak_search_log,
+        map_coeffs     = self.map_coeff).peaks_mapped()
       peaks.sites = fmodel.xray_structure.unit_cell().orthogonalize(peaks.sites)
       holes = find_peaks.manager(
-        fmodel=fmodel,
-        map_type=None,
-        map_coeffs=self.map_coeff,
-        map_cutoff=-self.map_cutoff,
-        params=self.peak_search_params,
-        log=peak_search_log).peaks_mapped()
+        map_cutoff     = -self.map_cutoff,
+        xray_structure = fmodel.xray_structure,
+        params         = self.peak_search_params,
+        log            = peak_search_log,
+        map_coeffs     = self.map_coeff).peaks_mapped()
       holes.sites = fmodel.xray_structure.unit_cell().orthogonalize(holes.sites)
       result = find_peaks_holes.peaks_holes_container(
         peaks=peaks,
