@@ -1188,7 +1188,7 @@ NOTES:
     # @todo reduce_hydrogens.py:run() says: TODO temporary fix until the code is moved to model class
     cs = self.model.crystal_symmetry()
     if (cs is None) or (cs.unit_cell() is None):
-      self.model = shift_and_box_model(model = self.model)
+      self.model = shift_and_box_model(model = self.model, shift_model=False)
 
     # If we've been asked to only to a single model index from the file, strip the model down to
     # only that index.
