@@ -175,7 +175,7 @@ def tst_01(log = sys.stdout):
 
   chainid_list = model_info.chainid_list
   print("Segments found: %s" %(" ".join(chainid_list)), file = log)
-  assert len(chainid_list) == 2
+  assert len(chainid_list) == 1
 
   # Check processing and splitting model into domains, adjusting domain size automatically
   print("\nProcessing and splitting model into domains", file = log)
@@ -225,13 +225,13 @@ def tst_01(log = sys.stdout):
 
   chainid_list = model_info.chainid_list
   print("Segments found: %s" %(" ".join(chainid_list)), file = log)
-  assert len(chainid_list) == 2
+  assert len(chainid_list) == 1
 
 
   mmm = model_info.model.as_map_model_manager()
   mmm.write_model('model_with_groupings.pdb')
   residue_count = []
-  expected_residue_count =  [84, 88]
+  expected_residue_count = [172]
   for chainid in chainid_list:
     selection_string = "chain %s" %(chainid)
     ph = model_info.model.get_hierarchy()
@@ -243,6 +243,7 @@ def tst_01(log = sys.stdout):
       selection_string, n),
        file = log)
     residue_count.append(n)
+  print(expected_residue_count,residue_count)
   assert expected_residue_count == residue_count
 
   # Now process and use pae model and pae model file
@@ -284,10 +285,19 @@ def tst_02(log = sys.stdout):
   print("Segments found: %s" %(" ".join(chainid_list)), file = log)
   assert len(chainid_list) == 2
 
+def tst_03():
+  from mmtbx.process_predicted_model import restore_true_except_at_ends
+  from scitbx.array_family import flex
+  a = flex.bool((0,0,0,1,0,1,1,0,0,0,1,1,1,0,0))
+  restore_true_except_at_ends(a)
+  print(list(a))
+  assert list(a) == [False, False, False, True, True, True, True, True, True, True, True, True, True, False, False]
+
 if __name__ == "__main__":
 
   t0 = time.time()
   tst_01()
   tst_02()
+  tst_03()
   print ("Time:", time.time()-t0)
   print ("OK")
