@@ -245,17 +245,6 @@ def condense_as_ranges(integer_array):
   store_range()
   return result
 
-@bp.inject_into(mersenne_twister)
-class _():
-
-  def random_selection(self, population_size, sample_size):
-    assert population_size >= 0
-    assert sample_size >= 0
-    assert sample_size <= population_size
-    perm = self.random_permutation(size=population_size)
-    perm.resize(sample_size)
-    return sorted(perm)
-
 random_generator = ext.mersenne_twister(scitbx.random.mt19937)
 
 def set_random_seed(value):
