@@ -10,17 +10,29 @@ parser.add_argument("inputGlob", type=str, help="glob of input pandas tables (th
 parser.add_argument("outdir", type=str, help="path to output refls")
 
 parser.add_argument("--cmdlinePhil", nargs="+", default=None, type=str, help="command line phil params")
+parser.add_argument("--dialsInteg", action="store_true", help="Integrate new shoeboxes using dials and write *integrated.expt files")
 parser.add_argument("--numdev", type=int, default=1, help="number of GPUs (default=1)")
 parser.add_argument("--pklTag", type=str, help="optional suffix for globbing for pandas pickles (default .pkl)", default=".pkl")
 parser.add_argument("--loud", action="store_true", help="show lots of screen output")
 parser.add_argument("--hopInputName", default="preds_for_hopper", type=str, help="write exp_ref_spec file and best_pickle pointing to the preditction models, such that one can run predicted rois through simtbx.diffBragg.hopper (e.g. to fit per-roi scale factors)")
 parser.add_argument("--filterDupes", action="store_true", help="filter refls with same HKL")
+parser.add_argument("--keepShoeboxes", action="store_true", help="Optionally keep shoeboxes present in the prediction refl tables (can lead to OOM errors)")
+parser.add_argument("--scanWeakFracs", action="store_true", help="optionally stores a variety of inputs for stage2 based filtering different fractions of weak reflections")
 
 args = parser.parse_args()
 
-from mpi4py import MPI
+from libtbx.mpi4py import MPI
 COMM = MPI.COMM_WORLD
 
+import logging
+if not args.loud:
+    logging.disable(logging.CRITICAL)
+else:
+    if COMM.rank==0:
+        logger = logging.getLogger("diffBragg.main")
+        logger.setLevel(logging.DEBUG)
+
+
 def printR(*args, **kwargs):
     print("RANK %d" % COMM.rank, *args, **kwargs)
 def print0(*args, **kwargs):
@@ -28,22 +40,40 @@ def print0(*args, **kwargs):
         print(*args, **kwargs)
 
 import numpy as np
-from simtbx.diffBragg import utils
+import json
+from simtbx.diffBragg import hopper_utils, utils
 from simtbx.modeling import predictions
 from simtbx.diffBragg.hopper_utils import downsamp_spec_from_params
 import glob
 import pandas
 import os
-import shutil
 from dials.algorithms.integration.stills_significance_filter import SignificanceFilter
 from dials.algorithms.indexing.stills_indexer import calc_2D_rmsd_and_displacements
-import logging
 import sys
 
-if not args.loud:
-    logging.disable(logging.CRITICAL)
-else:
-    logging.basicConfig(level=logging.DEBUG)
+
+def filter_weak_reflections(refls, weak_fraction):
+    """
+    :param pred:  reflection table created by this script
+    :param weak_fraction: number from 0-1 (if 0, only strong spots are saved)
+    :return: new reflection table with weak reflections filtered according to weak_fraction
+    """
+    new_refls = None
+    for idx in set(refls['id']):
+        pred = refls.select(refls['id']==idx)
+        weaks = pred.select(pred['is_weak'])
+        nweak = len(weaks)
+        weaks_sorted = np.argsort(weaks["scatter"])[::-1]
+        num_keep = int(nweak * weak_fraction)
+        weak_refl_inds_keep = set(np.array(weaks["refl_idx"])[weaks_sorted[:num_keep]])
+        weak_sel = flex.bool([i in weak_refl_inds_keep for i in pred['refl_idx']])
+        keeps = np.logical_or(pred['is_strong'], weak_sel)
+        pred = pred.select(flex.bool(keeps))
+        if new_refls is None:
+            new_refls = deepcopy(pred)
+        else:
+            new_refls.extend(pred)
+    return new_refls
 
 
 # Note: these imports and following 3 methods will eventually be in CCTBX/simtbx/diffBragg/utils
@@ -61,6 +91,7 @@ from dials.array_family import flex
 from copy import deepcopy
 from collections import Counter
 
+from simtbx.diffBragg.device import DeviceWrapper
 
 def filter_refls(R):
     vec3_dbl_keys = 'xyzcal.px', 'xyzcal.mm', 'xyzobs.px.value', 'xyzobs.px.value', 'rlp', 's1'
@@ -338,148 +369,248 @@ if __name__=="__main__":
             os.makedirs(args.outdir)
     COMM.barrier()
 
+    #rank_outdir = os.path.join( args.outdir, "rank%d" % COMM.rank)
+    #if not os.path.exists(rank_outdir):
+    #    os.makedirs(rank_outdir)
+
     params = utils.get_extracted_params_from_phil_sources(args.predPhil, args.cmdlinePhil)
-    if os.path.isfile(args.inputGlob):
-        df_all = pandas.read_pickle(args.inputGlob)
-        df_all.reset_index(inplace=True, drop=True)
-        def df_iter():
-            for i_f in range(len(df_all)):
-                if i_f % COMM.size != COMM.rank:
-                    continue
-                df_i = df_all.iloc[i_f:i_f+1].copy().reset_index(drop=True)
-                yield i_f, df_i
-        Nf = len(df_all)
-    else:
-        if os.path.isdir(args.inputGlob):
-            glob_s = os.path.join(args.inputGlob, "pandas/rank*/*.pkl")
-            fnames = glob.glob(glob_s)
+
+    # inputGlob can be a glob in strings, a single pandas file, or a hopper output folder
+    if os.path.isfile(args.inputGlob) or os.path.isdir(args.inputGlob):
+        if os.path.isfile(args.inputGlob):
+            fnames = [args.inputGlob]
         else:
-            fnames = glob.glob(args.inputGlob)
-        def df_iter():
-            for i_f,f in enumerate(fnames):
-                if i_f % COMM.size != COMM.rank:
-                    continue
-                df = pandas.read_pickle(f)
-                yield i_f, df
-        Nf = len(fnames)
+            dirname = args.inputGlob
+            fnames = glob.glob( os.path.join(dirname, "pandas/hopper_results_rank*.pkl"))
+    else:
+        fnames = glob.glob(args.inputGlob)
+
+    if not fnames:
+        raise OSError("Found no filenames to load!")
+    Nf = 0
+    shots_per_df = []
+    print0("getting total number of shots")
+    for i_f, f in enumerate(fnames):
+        if i_f % COMM.size != COMM.rank:
+            continue
+        n = len(pandas.read_pickle(f))
+        shots_per_df += [(f, str(x)) for x in range(n)]  # note we cast to string because of mpi reduce
+        Nf += n
+
+    shots_per_df = COMM.bcast(COMM.reduce( shots_per_df))
+    Nf = COMM.bcast(COMM.reduce(Nf))
+    print0("total num shots is %d" % Nf)
+    df_rows_per_rank = np.array_split(shots_per_df, COMM.size)[COMM.rank]
+
+    print0("getting dataframe handles")
+    df_handles = {}
+    dfs = []
+    if df_rows_per_rank.size:
+        dfs, _ = zip(*df_rows_per_rank)
+    for f in set(dfs):
+        df_handles[f] = pandas.read_pickle(f).reset_index(drop=True)
+
+    def df_iter():
+        for i_df, (df_name, row_idx) in enumerate(df_rows_per_rank):
+            row_idx = int(row_idx)
+            printR("Opening shot %d / %d" % (i_df+1, len(df_rows_per_rank)))
+            df = df_handles[df_name].iloc[row_idx: row_idx+1].copy()
+            yield i_df, df
 
     if params.predictions.verbose:
         params.predictions.verbose = COMM.rank==0
 
     dev = COMM.rank % args.numdev
 
-    print0("Found %d input files" % Nf)
-
-    all_dfs = []
-    all_pred_names = []
-    exp_ref_spec_lines = []
-    for i_f, df in df_iter():
-        printR("Shot %d / %d" % (i_f+1, Nf), flush=True)
-
-        expt_name = df.opt_exp_name.values[0]
-        tag = os.path.splitext(os.path.basename(expt_name))[0]
-        new_expt_name = "%s/%s_%d_predicted.expt" % (args.outdir,tag,  i_f)
-        new_expt_name = os.path.abspath(new_expt_name)
-        df["opt_exp_name"] = new_expt_name
-
-        shutil.copyfile(expt_name,  new_expt_name)
-
-        data_exptList = ExperimentList.from_file(expt_name)
-        data_expt = data_exptList[0]
-
-        try:
-            spectrum_override = None
-            if params.spectrum_from_imageset:
-                spectrum_override = downsamp_spec_from_params(params, data_expt)
-            pred = predictions.get_predicted_from_pandas(
-                df, params, strong=None, device_Id=dev, spectrum_override=spectrum_override)
-            if args.filterDupes:
-                pred = filter_refls(pred)
-        except ValueError:
-            os.remove(new_expt_name)
-            continue
-
-        data = utils.image_data_from_expt(data_expt)
-        Rstrong = refls_from_sims(data, data_expt.detector, data_expt.beam, phil_file=args.procPhil )
-        Rstrong['id'] = flex.int(len(Rstrong), 0)
-        num_panels = len(data_expt.detector)
-        if num_panels > 1:
-            assert params.predictions.label_weak_col == "rlp"
-
-        Rstrong.centroid_px_to_mm(data_exptList)
-        Rstrong.map_centroids_to_reciprocal_space(data_exptList)
-        predictions.label_weak_predictions(pred, Rstrong, q_cutoff=params.predictions.qcut, col=params.predictions.label_weak_col )
-
-        pred['is_strong'] = flex.bool(np.logical_not(pred['is_weak']))
-        strong_sel = np.logical_not(pred['is_weak'])
-
-        pred["refl_idx"] = flex.int(np.arange(len(pred)))
-        weaks = pred.select(pred['is_weak'])
-        weaks_sorted = np.argsort(weaks["scatter"])[::-1]
-        nweak = len(weaks)
-        num_keep = int(nweak*params.predictions.weak_fraction)
-        weak_refl_inds_keep = set(np.array(weaks["refl_idx"])[weaks_sorted[:num_keep]])
-
-        weak_sel = flex.bool([i in weak_refl_inds_keep for i in pred['refl_idx']])
-        keeps = np.logical_or( pred['is_strong'], weak_sel)
-        #printR("Sum keeps=%d; num_strong=%d, num_kept_weak=%d" % (sum(keeps), sum(strong_sel), sum(weak_sel)))
-        pred = pred.select(flex.bool(keeps))
-        nstrong = np.sum(strong_sel)
-        printR("Will save %d refls (%d strong, %d weak)" % (len(pred), np.sum(strong_sel), np.sum(weak_sel)))
-        pred_file = os.path.abspath("%s/%s_%d_predicted.refl" % ( args.outdir, tag, i_f))
-        pred.as_file(pred_file)
-
-        Rindexed = Rstrong.select(Rstrong['indexed'])
-        if len(Rindexed)==0:
-            print("No strong indexed refls for shot %s" % new_expt_name)
-            continue
-
-        utils.refls_to_hkl(Rindexed, data_expt.detector, data_expt.beam, data_expt.crystal, update_table=True)
-        try:
-            int_expt, int_refl = integrate(args.procPhil, data_exptList, Rindexed, pred)
-            int_expt_name = "%s/%s_%d_integrated.expt" % ( args.outdir,tag, i_f)
-            int_expt.as_file(int_expt_name)
-            int_refl['bbox'] = int_refl['shoebox'].bounding_boxes()
-            int_refl_name = int_expt_name.replace(".expt", ".refl")
-            int_refl.as_file(int_refl_name)
-        except RuntimeError as err:
-            print("Integration failed for %s because: '%s'" % (pred_file, str(err)))
-
-        all_dfs.append(df)
-        all_pred_names.append(pred_file)
-        spec_name = df.spectrum_filename.values[0]
-        if spec_name is None:
-            spec_name = ""
-        exp_ref_spec_lines.append("%s %s %s\n" % (new_expt_name, pred_file, spec_name))
-
-    if all_dfs:
-        all_dfs = pandas.concat(all_dfs)
-        all_dfs["predicted_refls"] = all_pred_names
-        all_dfs["predictions"] = all_pred_names
-    else:
-        all_dfs = None
-
-    all_dfs = COMM.gather(all_dfs)
-    exp_ref_spec_lines = COMM.reduce(exp_ref_spec_lines)
-    print0("\nReflections written to folder %s.\n" % args.outdir)
+    EXPT_DIRS = os.path.join(args.outdir, "expts_and_refls")
     if COMM.rank==0:
-        hopper_input_name = os.path.abspath(os.path.join(args.outdir , "%s.txt" % args.hopInputName))
-        o = open(hopper_input_name, "w")
-        for l in exp_ref_spec_lines:
-            o.write(l)
-        o.close()
-        all_dfs = [df for df in all_dfs if df is not None]
-        if not all_dfs:
-            raise ValueError("No dataframes to concat: prediction/integration failed for all shots..")
+        utils.safe_makedirs(EXPT_DIRS)
 
-        all_dfs = pandas.concat([df for df in all_dfs if df is not None])
-        all_dfs.reset_index(inplace=True, drop=True)
-        best_pkl_name = os.path.abspath(os.path.join(args.outdir , "%s.pkl" % args.hopInputName))
-        all_dfs.to_pickle(best_pkl_name)
-        print("Wrote %s (best_pickle option for simtbx.diffBragg.hopper) and %s (exp_ref_spec option for simtbx.diffBragg.hopper). Use them to run the predictions through hopper. Use the centroid=cal option to specify the predictions" % (best_pkl_name, hopper_input_name))
+    if args.scanWeakFracs and params.predictions.weak_fraction != 1:
+        print("WARNING: overriding weak_fracion because of scanWeakFracs")
+        params.predictions.weak_fraction=1
 
-        with open(args.outdir +".exectution.txt", "w") as o:
-            o.write("integrate was run from folder: %s\n" % os.getcwd())
-            o.write("The command line input was:\n")
-            o.write(" ".join(sys.argv))
-            #TODO: write the diff phils here:
+    print0("Found %d input files" % Nf)
+    with DeviceWrapper(dev) as _:
+        all_dfs = []
+        all_pred_names = []
+        exp_ref_spec_lines = []
+        all_rank_pred = None
+        all_rank_expt = None
+
+        rank_shot_count = 0
+        rank_pred_file = os.path.join(EXPT_DIRS, "rank%d_preds.refl" % COMM.rank)
+        rank_pred_file = os.path.abspath(rank_pred_file)
+        rank_expt_file = rank_pred_file.replace(".refl", ".expt")
+        for i_f, df in df_iter():
+
+            expt_name = df.exp_name.values[0]
+            expt_idx = df.exp_idx.values[0]
+            tag = os.path.splitext(os.path.basename(expt_name))[0]
+
+            data_expt = hopper_utils.DataModeler.exper_json_single_file(expt_name, expt_idx)
+            data_exptList = ExperimentList()
+            data_exptList.append(data_expt)
+
+            try:
+                spectrum_override = None
+                if params.spectrum_from_imageset:
+                    spectrum_override = downsamp_spec_from_params(params, data_expt)
+                pred = predictions.get_predicted_from_pandas(
+                    df, params, strong=None, device_Id=dev, spectrum_override=spectrum_override)
+                if args.filterDupes:
+                    pred = filter_refls(pred)
+            except ValueError:
+                #os.remove(new_expt_name)
+                continue
+
+            data = utils.image_data_from_expt(data_expt)
+            Rstrong = refls_from_sims(data, data_expt.detector, data_expt.beam, phil_file=args.procPhil )
+            Rstrong['id'] = flex.int(len(Rstrong), 0)
+            num_panels = len(data_expt.detector)
+            if num_panels > 1:
+                assert params.predictions.label_weak_col == "rlp"
+
+            Rstrong.centroid_px_to_mm(data_exptList)
+            Rstrong.map_centroids_to_reciprocal_space(data_exptList)
+            predictions.label_weak_predictions(pred, Rstrong, q_cutoff=params.predictions.qcut, col=params.predictions.label_weak_col )
+
+            pred['is_strong'] = flex.bool(np.logical_not(pred['is_weak']))
+            strong_sel = np.logical_not(pred['is_weak'])
+
+            pred["refl_idx"] = flex.int(np.arange(len(pred)))
+
+            #weaks = pred.select(pred['is_weak'])
+            #weaks_sorted = np.argsort(weaks["scatter"])[::-1]
+            #nweak = len(weaks)
+            #num_keep = int(nweak*params.predictions.weak_fraction)
+            #weak_refl_inds_keep = set(np.array(weaks["refl_idx"])[weaks_sorted[:num_keep]])
+            #weak_sel = flex.bool([i in weak_refl_inds_keep for i in pred['refl_idx']])
+            #keeps = np.logical_or( pred['is_strong'], weak_sel)
+            #printR("Sum keeps=%d; num_strong=%d, num_kept_weak=%d" % (sum(keeps), sum(strong_sel), sum(weak_sel)))
+            #pred = pred.select(flex.bool(keeps))
+            pred = filter_weak_reflections(pred, weak_fraction=params.predictions.weak_fraction)
+
+            nstrong = np.sum(strong_sel)
+            printR("Will save %d refls (%d strong, %d weak)" % (len(pred), np.sum(pred["is_strong"]), np.sum(pred["is_weak"])))
+            pred['id'] = flex.int(len(pred), rank_shot_count)
+            if 'shoebox' in list(pred) and not args.keepShoeboxes:
+                del pred['shoebox']
+            if all_rank_pred is None:
+                all_rank_pred = deepcopy(pred)
+            else:
+                all_rank_pred.extend(pred)
+
+            # Note, the simple append causes memory leak:
+            #all_rank_expt.append(data_expt)
+            if all_rank_expt is None:
+                all_rank_expt = deepcopy(data_exptList.to_dict())
+            else:
+                Edict = data_exptList.to_dict()
+                for exp_key in 'beam', 'detector', 'crystal', 'imageset':
+                    Edict['experiment'][0][exp_key] = rank_shot_count
+                for exp_key in 'experiment', 'beam', 'detector', 'crystal', 'imageset':
+                    assert len( Edict[exp_key])==1
+                    all_rank_expt[exp_key] .append(Edict[exp_key][0])
+
+            Rindexed = Rstrong.select(Rstrong['indexed'])
+            if len(Rindexed)==0:
+                print("No strong indexed refls for shot %s" % expt_name)
+                continue
+
+            utils.refls_to_hkl(Rindexed, data_expt.detector, data_expt.beam, data_expt.crystal, update_table=True)
+            if args.dialsInteg:
+                # TODO: save these files as multi-shot experiment/refls
+                try:
+                    int_expt, int_refl = integrate(args.procPhil, data_exptList, Rindexed, pred)
+                    int_expt_name = "%s/%s_%d_integrated.expt" % (rank_outdir, tag, i_f)
+                    int_expt.as_file(int_expt_name)
+                    int_refl['bbox'] = int_refl['shoebox'].bounding_boxes()
+                    int_refl_name = int_expt_name.replace(".expt", ".refl")
+                    int_refl.as_file(int_refl_name)
+                except RuntimeError:
+                    print("Integration failed" )
+
+            df['old_exp_name'] = expt_name
+            df['old_exp_idx'] = expt_idx
+            df['exp_name'] = rank_expt_file
+            df['exp_idx'] = rank_shot_count
+
+            df['predictions'] = rank_pred_file
+            df['predicted_refs'] = rank_pred_file
+            df['num_pred'] = len(pred)
+
+            all_dfs.append(df)
+            rank_shot_count += 1
+
+            spec_name = df.spectrum_filename.values[0]
+            if spec_name is None:
+                spec_name = ""
+            exp_ref_spec_lines.append("%s %s %s %d\n" % (rank_expt_file, rank_pred_file, spec_name, rank_shot_count))
+
+        all_rank_pred.as_file(rank_pred_file)
+        # NOTE: all_rank_expt is a dictionary to avoid weird OOM, so we write a simple json
+        #all_rank_expt.as_file(rank_expt_file)
+        with open(rank_expt_file, "w") as file_O:
+            json.dump(all_rank_expt, file_O)
+        print0("Done with predictions, combining dataframes")
+        if all_dfs:
+            all_dfs = pandas.concat(all_dfs)
+        else:
+            all_dfs = None
+
+        if args.scanWeakFracs and all_dfs is not None:
+            assert len(all_dfs.predictions.unique()) == 1
+            pred_file = all_dfs.predictions.values[0]
+            n_total_weak = np.sum(all_rank_pred['is_weak'])
+            n_total = len(all_rank_pred)
+            weak_fracs = [.11,.22,.33,.44,.55,.66,.77,.88]
+            labels = []
+            for i_frac, weak_frac in enumerate(weak_fracs):
+                filt_refls = filter_weak_reflections(all_rank_pred, weak_frac)
+                label="%dperc"%(weak_frac*100,)
+                labels.append(label)
+                new_pred_file = os.path.splitext(pred_file)[0]+"_%s.refl" % label
+                all_dfs['predictions_%s' % label] = new_pred_file
+                all_dfs['predicted_refs_%s' %label] = new_pred_file
+                num_preds = []
+                for exp_id in all_dfs.exp_idx.values:
+                    n = np.sum(filt_refls['id'] == int(exp_id))
+                    num_preds.append(n)
+                all_dfs['num_pred_%s' %label] = num_preds
+                filt_refls.as_file(new_pred_file)
+                printR("Saved %d/%d refls (%d strong, %d/%d weak) to %s"
+                       % (len(filt_refls), n_total, np.sum(filt_refls["is_strong"]), np.sum(filt_refls["is_weak"]), n_total_weak, new_pred_file))
+            # note this sanity check below requires that weaK_fracs be sorted
+            if sorted(weak_fracs) == weak_fracs:
+                # then as weak frac increases, there should be an increasing number of predictions
+                num_preds_per_frac = [all_dfs["num_pred_%s" % lab].sum() for lab in labels]
+                assert num_preds_per_frac == sorted(num_preds_per_frac)
+
+        print0("MPI gather all_dfs")
+        all_dfs = COMM.gather(all_dfs)
+        print0("MPI reduce lines")
+        exp_ref_spec_lines = COMM.reduce(exp_ref_spec_lines)
+        if COMM.rank==0:
+            hopper_input_name = os.path.abspath(os.path.join(args.outdir , "%s.txt" % args.hopInputName))
+            o = open(hopper_input_name, "w")
+            for l in exp_ref_spec_lines:
+                o.write(l)
+            o.close()
+            all_dfs = [df for df in all_dfs if df is not None]
+            if not all_dfs:
+                raise ValueError("No dataframes to concat: prediction/integration failed for all shots..")
+
+            print("Concat frames")
+            all_dfs = pandas.concat([df for df in all_dfs if df is not None])
+            all_dfs.reset_index(inplace=True, drop=True)
+            best_pkl_name = os.path.abspath(os.path.join(args.outdir , "%s.pkl" % args.hopInputName))
+            all_dfs.to_pickle(best_pkl_name)
+            print("Wrote %s (best_pickle option for simtbx.diffBragg.hopper) and %s (exp_ref_spec option for simtbx.diffBragg.hopper). Use them to run the predictions through hopper (use phil centroid=cal) or simtbx.diffBragg.stage_two." % (best_pkl_name, hopper_input_name))
+
+            cmd_log_file = os.path.join(args.outdir, "cmdline_execution.txt")
+            with open(cmd_log_file, "w") as o:
+                o.write("integrate was run from folder: %s\n" % os.getcwd())
+                o.write("The command line input was:\n")
+                o.write(" ".join(sys.argv) + "\n")
+                #TODO: write the diff phils here:
