@@ -15,12 +15,12 @@ Utility functions used for reading input data
 def create_experiment_identifier(experiment, experiment_file_path, experiment_id):
   'Create a hashed experiment identifier based on the experiment file path, experiment index in the file, and experiment features'
   import hashlib
-  exp_identifier_str = experiment_file_path + \
+  exp_identifier_str = os.path.basename(experiment_file_path + \
                        str(experiment_id) + \
                        str(experiment.beam) + \
                        str(experiment.crystal) + \
                        str(experiment.detector) + \
-                       ''.join(experiment.imageset.paths())
+                       ''.join([os.path.basename(p) for p in experiment.imageset.paths()]))
   hash_obj = hashlib.md5(exp_identifier_str.encode('utf-8'))
   return hash_obj.hexdigest()
 
@@ -115,7 +115,6 @@ class simple_file_loader(worker):
     if self.mpi_helper.rank == 0:
       file_list = list_input_pairs(self.params)
       self.logger.log("Built an input list of %d json/pickle file pairs"%(len(file_list)))
-      self.params.input.path = None # Rank 0 has already parsed the input parameters
 
       # optionally write a file list mapping to disk, useful in post processing if save_experiments_and_reflections=True
       file_id_from_names = None
