@@ -21,6 +21,8 @@ from six.moves import zip
 
 from xfel.command_line.experiment_manager import initialize as initialize_base
 
+CACHED_CONNECT_TIMEOUT = 300
+
 class initialize(initialize_base):
   expected_tables = ["run", "job", "rungroup", "trial", "tag", "run_tag", "event", "trial_rungroup",
                      "imageset", "imageset_event", "beam", "detector", "experiment",
@@ -281,10 +283,19 @@ class initialize(initialize_base):
       columns_dict[table_name] = [c[0] for c in cursor.fetchall() if c[0] != 'id']
     return columns_dict
 
+class dummy_cursor(object):
+  def __init__(self, sql_cursor):
+    self.rowcount = sql_cursor.rowcount
+    self.lastrowid = sql_cursor.lastrowid
+    self.prefetched = sql_cursor.fetchall()
+  def fetchall(self):
+    return self.prefetched
+
 class db_application(object):
-  def __init__(self, params, cache_connection = False, mode = 'execute'):
+  def __init__(self, params, cache_connection = True, mode = 'execute'):
     self.params = params
     self.dbobj = None
+    self.dbobj_refreshed_time = None
     self.cache_connection = cache_connection
     self.query_count = 0
     self.mode = mode
@@ -295,16 +306,15 @@ class db_application(object):
       assert val in ['execute', 'cache_commits']
     return super(db_application, self).__setattr__(prop, val)
 
-  def execute_query(self, query, commit = False):
+  def execute_query(self, query, commit=True):
     from MySQLdb import OperationalError
-    from time import time
 
     if self.mode == 'cache_commits' and commit:
       self.last_query = query
       return
 
     if self.params.db.verbose:
-      st = time()
+      st = time.time()
       self.query_count += 1
 
     retry_count = 0
@@ -312,24 +322,49 @@ class db_application(object):
     sleep_time = 0.1
     while retry_count < retry_max:
       try:
-        if self.dbobj is None:
-          dbobj = get_db_connection(self.params)
+
+        # Get the (maybe cached) connection
+        # We enable autocommit on the connection by default, to avoid stale
+        # reads arising from unclosed transactions. See:
+        # https://stackoverflow.com/questions/1617637/pythons-mysqldb-not-getting-updated-row
+        if not commit: # connection caching is not attempted if commit=False
+          dbobj = get_db_connection(self.params, autocommit=False)
+        elif (
+            self.dbobj is None
+            or time.time() - self.dbobj_refreshed_time > CACHED_CONNECT_TIMEOUT
+        ):
+          dbobj = get_db_connection(self.params, autocommit=True)
+          self.dbobj_refreshed_time = time.time()
           if self.cache_connection:
             self.dbobj = dbobj
         else:
           dbobj = self.dbobj
-        cursor = dbobj.cursor()
-        cursor.execute(query)
-        if commit:
-          dbobj.commit()
+
+        sql_cursor = dbobj.cursor()
+        sql_cursor.execute(query)
+        cursor = dummy_cursor(sql_cursor)
+        sql_cursor.close()
 
         if self.params.db.verbose:
-          et = time() - st
+          et = time.time() - st
           if et > 1:
             print('Query % 6d SQLTime Taken = % 10.6f seconds' % (self.query_count, et), query[:min(len(query),145)])
         return cursor
       except OperationalError as e:
-        if "Can't connect to MySQL server" not in str(e) and "Lost connection to MySQL server" not in str(e):
+        reconnect_strings = [
+            "MySQL server has gone away",
+            "max_user_connections",
+            "is not allowed to connect to this MariaDB server",
+        ]
+        retry_strings = [
+            "Can't connect to MySQL server",
+            "Lost connection to MySQL server",
+            "Deadlock found when trying to get lock",
+            "WSREP has not yet prepared node for application use",
+        ]
+        if any([s in str(e) for s in reconnect_strings]):
+          self.dbobj = None
+        elif all([s not in str(e) for s in retry_strings]):
           print(query)
           raise e
         retry_count += 1
@@ -809,8 +844,8 @@ class xfel_db_application(db_application):
   def get_all_jobs(self, active = False, where = None):
     if active:
       if where is None:
-        where = ""
-      where += " WHERE (trial.active = True AND rungroup.active = True) OR dataset.active = True"
+        where = "LEFT OUTER JOIN `%s_dataset_task` dataset_task ON job.dataset_id = dataset_task.dataset_id " % self.params.experiment_tag
+        where += "WHERE (trial.active = True AND rungroup.active = True) OR (dataset.active = True AND job.task_id = dataset_task.task_id AND dataset_task.task_id IS NOT NULL)"
     return self.get_all_x_with_subitems(JobFactory.from_args, "job", sub_items = [(Trial, 'trial', False),
                                                                                   (Run, 'run', False),
                                                                                   (Rungroup, 'rungroup', False),
@@ -886,9 +921,6 @@ class xfel_db_application(db_application):
   def create_dataset_version(self, **kwargs):
     return DatasetVersion(self, **kwargs)
 
-  def get_dataset(self, dataset_version_id):
-    return Dataset(self, dataset_version_id)
-
   def get_dataset_version(self, dataset_version_id):
     return DatasetVersion(self, dataset_version_id)
 
